import initSqlJs from 'sql.js';
import { useState, useCallback } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const createQueryFromSchema = (schema) => {
    const tables = Object.keys(schema);
    const createTable = "CREATE TABLE";
    let query = "";
    tables.forEach(table => {
        const { fields, values } = schema[table];
        const fieldsPart = Object.keys(fields).reduce((acc, field, idx, keys) => {
            return acc + `${field} ${fields[field]}` + (idx < keys.length - 1 ? ', ' : '');
        }, '');
        const insertPart = values
            ? values.map((data) => {
                const dataPart = Object.keys(data).reduce((acc, key, idx, keys) => {
                    const value = data[key];
                    return acc += (typeof value === "string" ? `'${value}'` : `${value}`) + ((idx < keys.length - 1 ? ', ' : ''));
                }, '');
                return `INSERT INTO ${table} VALUES (${dataPart});`;
            }).join('')
            : [];
        query += `${createTable} ${table} (${fieldsPart});${insertPart}`;
    });
    return [query, tables];
};
const database = {
    instance: null,
    setInstance: function (db) {
        this.instance = db;
    },
    getInstance: function () {
        return this.instance;
    },
    destroy: function () {
        this.instance = null;
    }
};
const tables = [];
const GlobalRecorder = {};
const createSQL = (path, schema) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const SQL = yield initSqlJs({
            locateFile: () => path
        });
        const db = new SQL.Database();
        database.setInstance(db);
        const [query, tablesList] = createQueryFromSchema(schema);
        tables.push(...tablesList);
        db.exec(query);
        return db;
    }
    catch (e) {
        console.error(e);
    }
    return;
});
const getDatabase = () => database.getInstance();
const readingQueryMatch = (query, tables) => {
    const readRegEx = new RegExp(`^(SELECT).+(?<table>${tables.join('|')})`);
    return query.match(readRegEx);
};
const writingQueryMatch = (query, tables) => {
    const readRegEx = new RegExp(`^(INSERT INTO).+(?<table>${tables.join('|')})`);
    return query.match(readRegEx);
};
const tableFromReadQuery = (query, tables) => {
    var _a;
    const regExRes = readingQueryMatch(query, tables);
    const table = (_a = regExRes === null || regExRes === void 0 ? void 0 : regExRes.groups) === null || _a === void 0 ? void 0 : _a.table;
    if (table) {
        return table;
    }
    return null;
};
const tableFromWritingQuery = (query, tables) => {
    var _a;
    const regExRes = writingQueryMatch(query, tables);
    const table = (_a = regExRes === null || regExRes === void 0 ? void 0 : regExRes.groups) === null || _a === void 0 ? void 0 : _a.table;
    if (table) {
        return table;
    }
    return null;
};
const registerQueryListeners = (updateState, query, tables, recorder) => {
    const table = tableFromReadQuery(query, tables);
    if (table) {
        if (!recorder[table]) {
            recorder[table] = [];
        }
        recorder[table].push(new WeakRef(updateState));
    }
};
const triggerActuators = (query, tables, recorder) => {
    const table = tableFromWritingQuery(query, tables);
    if (table) {
        const timestamp = Date.now();
        if (!recorder[table]) {
            return;
        }
        new Promise((resolve, reject) => {
            try {
                recorder[table].forEach(weakRef => {
                    const fnRef = weakRef.deref();
                    fnRef && fnRef(timestamp);
                });
                resolve(1);
            }
            catch (err) {
                console.error('UPDATE ERROR: ', err);
                reject();
            }
        });
    }
};
const executeQuery = (query, params) => {
    let db = getDatabase();
    if (!db) {
        console.error('SQL-lite instance not initiated! \nRun createSQL function to initialize the service.');
        return;
    }
    try {
        return db.exec(query, params);
        // @ts-ignore
    }
    catch (error) {
        console.error(`SQL-lite Error: ${error.message}`);
        return;
    }
};
const queryPipeline = (updateStateFn, query, params) => {
    registerQueryListeners(updateStateFn, query, tables, GlobalRecorder);
    const result = executeQuery(query, params);
    triggerActuators(query, tables, GlobalRecorder);
    return result;
};
const insertQueryPipeline = (updateStateFn, query, params, recorder = GlobalRecorder) => {
    const regExRes = writingQueryMatch(query, tables);
    if (!regExRes) {
        return;
    }
    const result = executeQuery(query, params);
    triggerActuators(query, tables, recorder);
    return result;
};

function useQuery(query, depths) {
    const [updatedAt, setUpdatedAt] = useState();
    const writeQueryFn = useCallback((query) => {
        insertQueryPipeline(setUpdatedAt, query);
    }, []);
    return {
        result: queryPipeline(setUpdatedAt, query),
        writeQueryFn
    };
}

export { createSQL as default, useQuery };
//# sourceMappingURL=index.js.map
